<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 */


namespace Amasty\Coupons\Helper;

use Magento\Framework\App\Area;
use Magento\Quote\Model\Quote\Address;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\SalesRule\Model\Coupon;
use Magento\SalesRule\Model\ResourceModel\Coupon as CouponResource;
use Amasty\Coupons\Model\CouponRenderer;
use Magento\Backend\Model\Session\Quote;
use Magento\Framework\App\State;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Data
 */
class Data
{
    /**
     * @var Coupon
     */
    private $couponModel;
    
    /**
     * @var CheckoutSession
     */
    private $session;

    /**
     * @var CouponResource
     */
    private $couponResource;

    /**
     * @var CouponRenderer
     */
    private $couponRenderer;

    /**
     * @var Quote
     */
    private $backendSession;

    /**
     * @var State
     */
    private $state;

    public function __construct(
        CheckoutSession $session,
        Coupon $couponModel,
        CouponResource $couponResourceModel,
        CouponRenderer $couponRenderer,
        Quote $backendSession,
        State $state
    ) {
        $this->couponModel = $couponModel;
        $this->session = $session;
        $this->couponResource = $couponResourceModel;
        $this->couponRenderer = $couponRenderer;
        $this->backendSession = $backendSession;
        $this->state = $state;
    }

    /**
     * @param bool $isRuleApplied
     * @param Address|null $address
     *
     * @return array|bool
     * @throws LocalizedException
     */
    public function getRealAppliedCodes($isRuleApplied = false, $address = null)
    {
        if ($address) {
            $quote = $address->getQuote();
        } else {
            $quote = $this->state->getAreaCode() === Area::AREA_ADMINHTML
            ? $this->backendSession->getQuote()
            : $this->session->getQuote();
        }

        if (!$quote->getCouponCode()) {
            return [];
        }

        $coupons = $this->couponRenderer->parseCoupon($quote->getCouponCode());
        $appliedRules = array_map('trim', explode(',', $quote->getAppliedRuleIds()));

        if (!$appliedRules) {
            return false;
        }

        foreach ($coupons as $key => $coupon) {
            $rule = $this->couponModel->loadByCode($coupon);
            if (!$this->couponResource->exists($coupon)
                || (!$isRuleApplied && !in_array($rule->getRuleId(), $appliedRules))
            ) {
                unset($coupons[$key]);
            }
        }

        $coupons = array_unique($coupons);

        return $coupons;
    }
}
