<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 */


namespace Amasty\Coupons\Observer;

use Magento\Framework\Event\ObserverInterface;
use Amasty\Coupons\Model\DiscountCollector;
use Magento\Framework\Event\Observer;

/**
 * Class CouponObserver
 */
class CouponObserver implements ObserverInterface
{
    /**
     * @var DiscountCollector
     */
    protected $discountCollector;

    /**
     * @var string
     */
    private $codeKey;

    public function __construct(
        DiscountCollector $discountCollector
    ) {
        $this->discountCollector = $discountCollector;
    }

    /**
     * event salesrule_validator_process
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        $appliedCodes = [];

        if ($observer->getData('quote')->getCouponCode()) {
            $appliedCodes = explode(',', $observer->getData('quote')->getCouponCode());
        }

        if (!$this->codeKey) {
            $this->codeKey = 0;
        }
        
        if (!isset($appliedCodes[$this->codeKey])) {
            return;
        }

        if ($observer->getData('rule')->getCouponCode() != null) {
            $this->discountCollector->applyRuleAmount(
                $appliedCodes[$this->codeKey],
                $observer->getData('result')->getAmount()
            );
            $this->codeKey++;
        } elseif ($observer->getData('rule')->getUseAutoGeneration()) {
            $this->discountCollector->applyRuleAmount(
                $appliedCodes[$this->codeKey],
                $observer->getData('result')->getAmount()
            );
            $this->codeKey++;
        }
    }
}
